@@ -1,21 +1,15 @@
-"""This module contains the base classes for simple and advanced actuators.
-These classes are the base contract for actuators.
-If you wish to create your own one, you need to inheritate from one of these base class.
-
-This module contains the simple actuators classes.
-Simple actuators are movement related one. They allow for predetermined movements
-patterns.
-
+__docformat__ = "restructuredtext"
+"""
 .. autosummary::
    :toctree: .
 
-    Actuator
-    Behavioral
-    RandomActuator
-    PathActuator
-    PatrolActuator
-    UnidirectionalActuator
-    PathFinder
+    pygamelib.actuators.Actuator
+    pygamelib.actuators.Behavioral
+    pygamelib.actuators.RandomActuator
+    pygamelib.actuators.PathActuator
+    pygamelib.actuators.PatrolActuator
+    pygamelib.actuators.UnidirectionalActuator
+    pygamelib.actuators.PathFinder
 
 """
 from pygamelib import board_items
@@ -23,9 +17,10 @@ from pygamelib import base
 from pygamelib import constants
 import random
 import collections
+from queue import PriorityQueue
 
 
-class Actuator:
+class Actuator(base.PglBaseObject):
     """
     Actuator is the base class for all Actuators. It is mainly a contract class with
     some utility methods.
@@ -45,6 +40,7 @@ class Actuator:
             actuator to be in a different state (PAUSED for example), you have to do it
             yourself.
         """
+        super().__init__()
         self.type = None
         self.state = constants.RUNNING
         self.parent = parent
@@ -88,6 +84,28 @@ class Actuator:
         """
         raise NotImplementedError()
 
+    def serialize(self):
+        """
+        Serializes the actuator and returns it as a dict.
+
+        That method needs to be implemented by all actuators or a NotImplementedError
+        exception will be raised.
+
+        :raises: NotImplementedError
+        """
+        raise NotImplementedError()
+
+    def load(self, data: dict = None):
+        """
+        Load serialized data, create and returns a new actuator out of these data.
+
+        That method needs to be implemented by all actuators or a NotImplementedError
+        exception will be raised.
+
+        :raises: NotImplementedError
+        """
+        raise NotImplementedError()
+
 
 class Behavioral(Actuator):
     """
@@ -101,10 +119,10 @@ class Behavioral(Actuator):
 
     def __init__(self, parent):
         """
-        The constructor simply construct an Actuator. It takes on positional paraneter:
+        The constructor simply construct an Actuator. It takes on positional parameter:
         the parent object.
         """
-        Actuator.__init__(self, parent)
+        super().__init__(parent)
 
     def next_action(self):
         """
@@ -133,23 +151,111 @@ class RandomActuator(Actuator):
         if moveset is None:
             moveset = []
         super().__init__(parent)
+        self.__moveset = []
+        self._vector_moveset = []
+        self.__current_direction = None
+        self.__current_dir_move_left = None
+        # We'll use that to check if the moving board item is stuck.
+        self.__projected_position_cache = None
         self.moveset = moveset
 
+    @property
+    def moveset(self):
+        """
+        Return the moveset.
+
+        :return: The moveset.
+        :rtype: list
+        """
+        return self.__moveset
+
+    @moveset.setter
+    def moveset(self, moveset):
+        """
+        Set the moveset.
+
+        :param moveset: The moveset.
+        :type moveset: list
+        """
+        self.__moveset = moveset
+        self._vector_moveset = []
+        if len(self.moveset) > 0:
+            # let's build a cache of directions to avoid creating vectors at each
+            # next_move() call.
+            for m in self.moveset:
+                if isinstance(m, base.Vector2D):
+                    self._vector_moveset.append(m)
+                else:
+                    # Here we consider that in moveset, there's either Vector2D or
+                    # directions from the constants module. If it is not the case,
+                    # result will be funky...
+                    self._vector_moveset.append(base.Vector2D.from_direction(m, 1))
+            self.__current_direction = random.randrange(0, len(self.moveset))
+            self.__current_dir_move_left = random.randint(1, 10)
+
     def next_move(self):
         """Return a randomly selected movement
 
         The movement is randomly selected from moveset if state is RUNNING,
-        otherwise it should return None.
+        otherwise it returns NO_DIR from the :py:mod:`~pygamelib.constants` module.
 
         :return: The next movement
-        :rtype: int | None
+        :rtype: int | :py:const:`pygamelib.constants.NO_DIR`
 
         Example::
 
-            randomactuator.next_move()
+            random_actuator.next_move()
         """
-        if self.state == constants.RUNNING:
-            return random.choice(self.moveset)
+        if self.state == constants.RUNNING and self.moveset:
+            ppav = None
+            if isinstance(self.parent, board_items.Movable):
+                ppav = self.parent.position_as_vector()
+            if (
+                self.__current_dir_move_left is None
+                or self.__current_dir_move_left <= 0
+                or ppav != self.__projected_position_cache
+            ):
+
+                self.__current_direction = random.randrange(0, len(self.moveset))
+                self.__current_dir_move_left = random.randint(1, 10)
+            self.__current_dir_move_left -= 1
+            if ppav is not None:
+                self.__projected_position_cache = (
+                    ppav + self._vector_moveset[self.__current_direction]
+                )
+            return self.moveset[self.__current_direction]
+
+            # return random.choice(self.moveset)
+        else:
+            return constants.NO_DIR
+
+    def serialize(self) -> dict:
+        """Return a dictionary with all the attributes of this object.
+
+        :return: A dictionary with all the attributes of this object.
+        :rtype: dict
+        """
+        return {"type": "RandomActuator", "moveset": self.moveset, "state": self.state}
+
+    @classmethod
+    def load(cls, data: dict = None):
+        """Load data and create a new RandomActuator out of it.
+
+        :param data: Data to create a new actuator (usually generated by
+           :meth:`serialize()`)
+        :type data: dict
+
+        :return: A new actuator.
+        :rtype: RandomActuator
+
+        Example::
+
+            npc2.actuator = actuators.RandomActuator.load( npc1.actuator.serialize() )
+        """
+        act = cls(moveset=data["moveset"])
+        if "state" in data.keys():
+            act.state = data["state"]
+        return act
 
 
 class PathActuator(Actuator):
@@ -179,16 +285,16 @@ class PathActuator(Actuator):
         """Return the movement based on current index
 
         The movement is selected from path if state is RUNNING, otherwise
-        it should return None. When state is RUNNING, the movement is selected
-        before incrementing the index by 1. When the index equal the length of
-        path, the index should return back to 0.
+        it returns NO_DIR from the :py:mod:`~pygamelib.constants` module. When state is
+        RUNNING, the movement is selected before incrementing the index by 1. When the
+        index equal the length of path, the index should return back to 0.
 
         :return: The next movement
-        :rtype: int | None
+        :rtype: int | :py:const:`pygamelib.constants.NO_DIR`
 
         Example::
 
-            pathactuator.next_move()
+            path_actuator.next_move()
         """
         if self.state == constants.RUNNING:
             move = self.path[self.index]
@@ -196,6 +302,8 @@ class PathActuator(Actuator):
             if self.index == len(self.path):
                 self.index = 0
             return move
+        else:
+            return constants.NO_DIR
 
     def set_path(self, path):
         """Defines a new path
@@ -207,19 +315,47 @@ class PathActuator(Actuator):
 
         Example::
 
-            pathactuator.set_path([constants.UP,constants.DOWN,constants.LEFT,constants.RIGHT])
+            path_actuator.set_path([constants.UP,constants.DOWN,constants.LEFT,constants.RIGHT])
         """
         self.path = path
         self.index = 0
 
+    def serialize(self) -> dict:
+        """Return a dictionary with all the attributes of this object.
+
+        :return: A dictionary with all the attributes of this object.
+        :rtype: dict
+        """
+        return {"type": "PathActuator", "path": self.path, "state": self.state}
+
+    @classmethod
+    def load(cls, data: dict = None):
+        """Load data and create a new PathActuator out of it.
+
+        :param data: Data to create a new actuator (usually generated by
+           :meth:`serialize()`)
+        :type data: dict
+
+        :return: A new actuator.
+        :rtype: PathActuator
+
+        Example::
+
+            path_actuator = PathActuator.load(actuator_data)
+        """
+        act = cls(path=data["path"])
+        if "state" in data.keys():
+            act.state = data["state"]
+        return act
+
 
 class PatrolActuator(PathActuator):
     """
     The patrol actuator is a subclass of
     :class:`~pygamelib.actuators.PathActuator`.  The move inside the function
     next_move depends on path and index and the mode. Once it reaches the end
-    of the move list it will start cycling back to the beggining of the list.
-    Once it reaches the beggining it will start moving forwards
+    of the move list it will start cycling back to the beginning of the list.
+    Once it reaches the beginning it will start moving forwards
     If the state is not running it returns None otherwise it increments the
     index & then, further compares the index with length of the path.
     If they both are same then, index is set to value zero and the move is
@@ -232,18 +368,18 @@ class PatrolActuator(PathActuator):
     def next_move(self):
         """Return the movement based on current index
 
-        The movement is selected from path if state is RUNNING, otherwise it
-        should return None. When state is RUNNING, the movement is selected
-        before incrementing the index by 1. When the index equals the length
-        of path, the index should return back to 0 and the path list should be
-        reversed before the next call.
+        The movement is selected from path if state is RUNNING, otherwise it returns
+        NO_DIR from the :py:mod:`~pygamelib.constants` module. When state is RUNNING,
+        the movement is selected before incrementing the index by 1. When the index
+        equals the length of path, the index should return back to 0 and the path list
+        should be reversed before the next call.
 
         :return: The next movement
-        :rtype: int | None
+        :rtype: int | :py:const:`pygamelib.constants.NO_DIR`
 
         Example::
 
-            patrolactuator.next_move()
+            patrol_actuator.next_move()
         """
         if self.state == constants.RUNNING:
             move = self.path[self.index]
@@ -269,6 +405,36 @@ class PatrolActuator(PathActuator):
                     elif self.path[i] == constants.DRUP:
                         self.path[i] = constants.DLDOWN
             return move
+        else:
+            return constants.NO_DIR
+
+    def serialize(self) -> dict:
+        """Return a dictionary with all the attributes of this object.
+
+        :return: A dictionary with all the attributes of this object.
+        :rtype: dict
+        """
+        return {"type": "PatrolActuator", "path": self.path, "state": self.state}
+
+    @classmethod
+    def load(cls, data: dict = None):
+        """Load data and create a new PatrolActuator out of it.
+
+        :param data: Data to create a new actuator (usually generated by
+           :meth:`serialize()`)
+        :type data: dict
+
+        :return: A new actuator.
+        :rtype: PatrolActuator
+
+        Example::
+
+            patrol_actuator = PatrolActuator.load(actuator_data)
+        """
+        act = cls(path=data["path"])
+        if "state" in data.keys():
+            act.state = data["state"]
+        return act
 
 
 class UnidirectionalActuator(Actuator):
@@ -295,10 +461,10 @@ class UnidirectionalActuator(Actuator):
         """Return the direction.
 
         The movement is always direction if state is RUNNING,
-        otherwise it returns None.
+        otherwise it returns NO_DIR from the :py:mod:`~pygamelib.constants` module.
 
         :return: The next movement
-        :rtype: int | None
+        :rtype: int | :py:const:`pygamelib.constants.NO_DIR`
 
         Example::
 
@@ -307,6 +473,38 @@ class UnidirectionalActuator(Actuator):
         if self.state == constants.RUNNING:
             return self.direction
 
+    def serialize(self) -> dict:
+        """Return a dictionary with all the attributes of this object.
+
+        :return: A dictionary with all the attributes of this object.
+        :rtype: dict
+        """
+        return {
+            "type": "UnidirectionalActuator",
+            "direction": self.direction,
+            "state": self.state,
+        }
+
+    @classmethod
+    def load(cls, data: dict = None):
+        """Load data and create a new UnidirectionalActuator out of it.
+
+        :param data: Data to create a new actuator (usually generated by
+           :meth:`serialize()`)
+        :type data: dict
+
+        :return: A new actuator.
+        :rtype: UnidirectionalActuator
+
+        Example::
+
+            unidir_actuator = UnidirectionalActuator.load(actuator_data)
+        """
+        act = cls(direction=data["direction"])
+        if "state" in data.keys():
+            act.state = data["state"]
+        return act
+
 
 class PathFinder(Behavioral):
     """
@@ -320,7 +518,7 @@ class PathFinder(Behavioral):
 
     The constructor takes the following parameters:
 
-    :param game: A reference to the instanciated game engine.
+    :param game: A reference to the instantiated game engine.
     :type game: pygamelib.engine.Game
     :param actuated_object: The object to actuate. Deprecated in favor of parent.
         Only kept for backward compatibility.
@@ -331,12 +529,20 @@ class PathFinder(Behavioral):
         method is going to circle between the waypoints
         (when the last is visited, go back to the first)
     :type circle_waypoints: bool
+    :param algorithm: ALGO_BFS - BFS, ALGO_ASTAR - AStar
+    :type algorithm: constant
 
     """
 
     def __init__(
-        self, game=None, actuated_object=None, circle_waypoints=True, parent=None
+        self,
+        game=None,
+        actuated_object=None,
+        circle_waypoints=True,
+        parent=None,
+        algorithm=constants.ALGO_BFS,
     ):
+        self.parent = None
         if actuated_object is not None and parent is None:
             self.actuated_object = actuated_object
             self.parent = actuated_object
@@ -350,6 +556,15 @@ class PathFinder(Behavioral):
         self.waypoints = []
         self._waypoint_index = 0
         self.circle_waypoints = circle_waypoints
+        self.algorithm = algorithm
+        if type(self.algorithm) is not int or (
+            self.algorithm != constants.ALGO_BFS
+            and self.algorithm != constants.ALGO_ASTAR
+        ):
+            raise base.PglInvalidTypeException(
+                "In Actuator.PathFinder.__init__(..,algorithm) algorithm must be"
+                "either ALGO_BFS or ALGO_ASTAR."
+            )
 
     def set_destination(self, row=0, column=0):
         """Set the targeted destination.
@@ -377,9 +592,7 @@ class PathFinder(Behavioral):
         """Find a path to the destination.
 
         Destination (PathFinder.destination) has to be set beforehand.
-        This method implements a Breadth First Search algorithm
-        (`Wikipedia <https://en.wikipedia.org/wiki/Breadth-first_search>`_)
-        to find the shortest path to destination.
+
 
         Example::
 
@@ -394,6 +607,18 @@ class PathFinder(Behavioral):
         .. warning:: PathFinder.destination is a tuple!
             Please use PathFinder.set_destination(x,y) to avoid problems.
 
+        Path Finding Algorithm Description:
+
+        Breadth First Search:
+        This method implements a Breadth First Search algorithm
+        (`Wikipedia: BFS <https://en.wikipedia.org/wiki/Breadth-first_search>`_)
+        to find the shortest path to destination.
+
+        A* Search:
+        This method implements a A* Search algorithm
+        (`Wikipedia: A* <https://en.wikipedia.org/wiki/A*_search_algorithm>`_)
+        to find the shortest path to destination.
+
         """
         if self.actuated_object is None:
             raise base.PglException(
@@ -411,7 +636,12 @@ class PathFinder(Behavioral):
                 "destination is not defined",
                 "PathFinder.destination has to be defined.",
             )
+        if self.algorithm == constants.ALGO_BFS:
+            return self.__find_path_bfs()
 
+        return self.__find_path_astar()
+
+    def __find_path_bfs(self):
         queue = collections.deque(
             [[(self.actuated_object.pos[0], self.actuated_object.pos[1])]]
         )
@@ -436,6 +666,52 @@ class PathFinder(Behavioral):
                     seen.add((r, c))
         return []
 
+    def __find_path_astar(self):
+
+        queue = PriorityQueue()
+
+        # queue stores a tuple with values:
+        # h - heuristic value = depth + manhattan distance from current node to
+        # destination
+        # type(h) = int
+        # path - path to reach current node from start node
+        # type(path) = list
+        # For each node, depth = len(path)
+
+        initial_h = abs(self.actuated_object.pos[0] - self.destination[0]) + abs(
+            self.actuated_object.pos[1] - self.destination[1]
+        )
+
+        queue.put(
+            (initial_h, [(self.actuated_object.pos[0], self.actuated_object.pos[1])])
+        )
+        seen = set()
+        while not queue.empty():
+            h_val, path = queue.get()
+            x, y = path[-1]
+            if (x, y) == self.destination:
+                self._current_path = path
+                # We return only a copy of the path as we need to keep the
+                # real one untouched for our own needs.
+                return path.copy()
+
+            # r = row c = column
+            for r, c in ((x + 1, y), (x - 1, y), (x, y + 1), (x, y - 1)):
+                h_val = (
+                    len(path)
+                    + abs(self.destination[0] - r)
+                    + abs(self.destination[1] - c)
+                )
+                if (
+                    0 <= c < self.game.current_board().size[0]
+                    and 0 <= r < self.game.current_board().size[1]
+                    and self.game.current_board().item(r, c).overlappable()
+                    and ((r, c) not in seen)
+                ):
+                    queue.put((h_val, path + [(r, c)]))
+                    seen.add((r, c))
+        return []
+
     def current_path(self):
         """This method simply return a copy of the current path of the actuator.
 
@@ -477,7 +753,7 @@ class PathFinder(Behavioral):
          - Look at the current waypoint, if the actuated object is not at \
             that position return a direction from the \
             :mod:`~pygamelib.constants` module. The direction is calculated \
-            from the difference betwen actuated object's position and \
+            from the difference between actuated object's position and \
             waypoint's position.
          - If the actuated object is at the waypoint position, then call \
             next_waypoint(), set the destination and return a direction. \
@@ -487,7 +763,7 @@ class PathFinder(Behavioral):
 
         Example::
 
-            seeker = NPC(model=Sprites.SKULL)
+            seeker = NPC(model=graphics.Models.SKULL)
             seeker.actuator = PathFinder(game=mygame,actuated_object=seeker)
             while True:
                 seeker.actuator.set_destination(mygame.player.pos[0],mygame.player.pos[1])
@@ -673,7 +949,7 @@ class PathFinder(Behavioral):
         return self.waypoints[self._waypoint_index]
 
     def remove_waypoint(self, row, column):
-        """ Remove a waypoint from the stack.
+        """Remove a waypoint from the stack.
 
         This method removes the first occurrence of a waypoint in the stack.
 
@@ -690,7 +966,7 @@ class PathFinder(Behavioral):
 
         Example::
 
-            method()
+            path_finder.remove_waypoint(2,5)
         """
         if type(row) is not int:
             raise base.PglInvalidTypeException('"row" is not an integer. It must be.')
@@ -706,3 +982,45 @@ class PathFinder(Behavioral):
                 "invalid_waypoint",
                 f"Waypoint ({row},{column}) does not exist in the waypoints stack.",
             )
+
+    def serialize(self) -> dict:
+        """Return a dictionary with all the attributes of this object.
+
+        :return: A dictionary with all the attributes of this object.
+        :rtype: dict
+        """
+        return {
+            "type": "PathFinder",
+            "waypoints": self.waypoints,
+            "destination": self.destination,
+            "circle_waypoints": self.circle_waypoints,
+            "algorithm": self.algorithm,
+            "state": self.state,
+        }
+
+    @classmethod
+    def load(cls, data: dict = None):
+        """Load data and create a new PathFinder out of it.
+
+        :param data: Data to create a new actuator (usually generated by
+           :meth:`serialize()`)
+        :type data: dict
+
+        :return: A new actuator.
+        :rtype: PathFinder
+
+        Example::
+
+            path_finder = PathFinder.load(actuator_data)
+        """
+        act = cls(
+            circle_waypoints=data["circle_waypoints"],
+            algorithm=data["algorithm"],
+        )
+        if "state" in data.keys():
+            act.state = data["state"]
+        if "waypoints" in data.keys():
+            act.waypoints = data["waypoints"]
+        if "destination" in data.keys():
+            act.destination = data["destination"]
+        return act
